from django.test import TestCase
from django.test.utils import override_settings

from mock import patch, Mock

from .management.commands.wtfd import Command
from . import MissingDocstringsException


TEST_APP_NAME = 'test_app'


class WTFDCommandTestCase(TestCase):
    """
    Tests for wtfd command.
    """

    def setUp(self):
        self.cmd = Command()

    @override_settings(WTFD_APPS=[TEST_APP_NAME])
    @override_settings(WTFD_STRICT=True)
    def test_init(self):
        cmd = Command()
        # Attributes exist
        self.assertTrue(hasattr(cmd, 'reports'))
        self.assertTrue(hasattr(cmd, 'apps'))
        self.assertTrue(hasattr(cmd, 'strict_mode'))
        # Attributes are valid
        self.assertEqual(cmd.reports, [])
        self.assertEqual(cmd.apps, [TEST_APP_NAME])
        self.assertTrue(cmd.strict_mode)

    def test_validate_filename(self):
        # Python file
        self.assertTrue(self.cmd._validate_filename('file.py'))
        # Not python
        self.assertFalse(self.cmd._validate_filename('file.jpg'))
        # Tests
        self.assertFalse(self.cmd._validate_filename('test.py'))
        self.assertFalse(self.cmd._validate_filename('models_tests.py'))
        self.assertFalse(self.cmd._validate_filename('test_views.py'))

    def test_valid_path(self):
        # Excludes migrations and tests
        self.assertFalse(self.cmd._valid_path('app/migrations'))
        self.assertFalse(self.cmd._valid_path('app/tests'))
        # Valid path
        self.assertTrue(self.cmd._valid_path('app/imports'))

    @patch('django_wtfd.management.commands.wtfd.Command._collect_mod_filenames')
    def test_collect_filenames_no_apps(self, mock_meth):
        self.cmd.apps = []
        self.cmd.collect_filenames()
        mock_meth.assert_called_once_with()

    @patch('django_wtfd.management.commands.wtfd.Command._collect_mod_filenames')
    def test_collect_filenames(self, mock_meth):
        self.cmd.apps = ['django', 'django_wtfd']
        self.cmd.collect_filenames()
        self.assertTrue(len(mock_meth.mock_calls) >= 2)

    @patch('django_wtfd.management.commands.wtfd.Command._collect_mod_filenames')
    def test_collect_filenames_invalid_app(self, mock_meth):
        self.cmd.apps = ['1d81h2d8h192dh2397dh2973h3d_____']
        self.cmd.collect_filenames()
        self.assertFalse(mock_meth.called)

    def test_collect_mod_filenames_syntax(self):
        collected = self.cmd._collect_mod_filenames()
        self.assertTrue(any(collected))

    def test_valid_node_invalid_class(self):
        name = 'noone_can_guess_this_name_srs'
        node = Mock(name=name)
        self.assertFalse(self.cmd._valid_node(node))

    def test_valid_node_valid_class(self):
        from ast import ClassDef, FunctionDef
        name = 'noone_can_guess_this_name_srs'
        self.assertTrue(self.cmd._valid_node(ClassDef(name=name)))
        self.assertTrue(self.cmd._valid_node(FunctionDef(name=name)))

    def test_valid_node_corner_cases(self):
        from ast import ClassDef, FunctionDef
        name = 'noone_can_guess_this_name_srs'
        self.assertTrue(self.cmd._valid_node(ClassDef(name=name)))
        self.assertTrue(self.cmd._valid_node(FunctionDef(name=name)))

    def test_store_report(self):
        node = Mock(name='Node', lineno=42)
        self.assertEqual(self.cmd.reports, [])
        self.cmd._store_report(node, '/tmp/')
        self.assertEqual(len(self.cmd.reports), 1)

    def test_check_docstrings_syntax(self):
        # TODO: Write more sane test.
        filename = self.cmd._collect_mod_filenames()[0]
        self.cmd.check_docstrings(filename)

    def test_report_missing_docstrings_no_reports(self):
        self.cmd.strict_mode = True
        self.cmd.reports = []
        try:
            self.cmd.report_missing_docstrings()
        except MissingDocstringsException:
            self.fail("report_missing_docstrings() raised "
                      "MissingDocstringsException unexpectedly!")

    def test_report_missing_docstrings_reports(self):
        self.cmd.strict_mode = True
        self.cmd.reports = ['asd']
        self.assertRaises(
            MissingDocstringsException,
            self.cmd.report_missing_docstrings,
        )

    @patch('django_wtfd.management.commands.wtfd.Command.collect_filenames')
    @patch('django_wtfd.management.commands.wtfd.Command.check_docstrings')
    @patch('django_wtfd.management.commands.wtfd.Command.report_missing_docstrings')
    def test_handle(self, mock_report, mock_check, mock_collect):
        mock_collect.return_value = ['asd']
        self.cmd.handle()
        mock_collect.assert_called_once_with()
        mock_check.assert_called_once_with('asd')
        mock_report.assert_called_once_with()



import os
import ast

from django.core.management.base import BaseCommand
from django.conf import settings

from ... import MissingDocstringsException


class Command(BaseCommand):
    """
    Parses Django project's modules and checks whether all of them
    have docstrings.
    """
    EXCLUDES = (
        'meta',
    )

    def __init__(self, *args, **kwargs):
        """
        Get WTFD_APPS setting from Django settings on init.
        """
        self.apps = getattr(settings, 'WTFD_APPS', [])
        self.strict_mode = getattr(settings, 'WTFD_STRICT', False)
        self.reports = []
        super(Command, self).__init__(*args, **kwargs)

    def _validate_filename(self, filename):
        """
        Checks filename to only fetch .py files & don't fetch tests.
        """
        is_pyfile = filename.endswith('.py')
        not_test = 'test' not in filename
        return (filename and is_pyfile and not_test)

    def _valid_path(self, path):
        """
        Check path so it won't contain migrations/tests directories.
        """
        not_migrations = not path.endswith('/migrations')
        not_tests = not path.endswith('/tests')
        return (not_migrations and not_tests)

    def collect_filenames(self):
        """
        Collects filenames from self.apps or all the project.
        """
        if not self.apps:
            return self._collect_mod_filenames()
        collected = []
        for app in self.apps:
            try:
                module = __import__(app.split('.')[-1])
            except ImportError:
                print 'Skipping not valid app: {}'.format(app)
                continue
            collected += self._collect_mod_filenames(module.__path__[0])
        return collected

    def _collect_mod_filenames(self, path='.'):
        """
        Collect filenames on given path.
        """
        collected = []
        for root, dirnames, filenames in os.walk(path):
            if not self._valid_path(root):
                continue
            filenames = filter(self._validate_filename, filenames)
            collected += [os.path.join(root, f) for f in filenames]
        return collected

    def _valid_node(self, node):
        """
        Checks if a node is either class or func and if it
        should be excluded from checks.
        """
        if not isinstance(node, (ast.ClassDef, ast.FunctionDef)):
            return False
        magic_method = node.name.startswith('__')
        in_excluded = node.name.lower() in self.EXCLUDES
        if magic_method or in_excluded:
            return False
        return True

    def _store_report(self, node, file_path):
        """
        Reports missing docstrings on a node.
        """
        schema = '\033[94m{}\033[0m \033[95m{}\033[0m line {}'
        report = schema.format(node.name, file_path, unicode(node.lineno))
        self.reports.append(report)

    def check_docstrings(self, file_path):
        """
        Parses the file located on :file_path: and
        prints a report if any class or method is missing
        docstringsin in it.
        """
        with open(file_path) as f:
            f_content = f.read()
        syntax_tree = ast.parse(f_content)

        for node in ast.walk(syntax_tree):
            if not self._valid_node(node):
                continue
            doc = ast.get_docstring(node)
            doc = None if doc is None else doc.strip()
            if not doc:
                self._store_report(node, file_path)

    def report_missing_docstrings(self):
        """
        Prints or raises report on missing docstrings.
        WTFD_STRICT == True  => Raises
        WTFD_STRICT == False => Prints
        """
        if not any(self.reports):
            return
        traceback = '\n'.join(self.reports)
        if self.strict_mode:
            raise MissingDocstringsException(traceback)
        print '\033[91mMissing docstrings:\033[0m'
        print traceback

    def handle(self, *args, **options):
        """
        High-level logic. Calls other methods.
        """
        files = self.collect_filenames()
        for file_path in files:
            self.check_docstrings(file_path)

        self.report_missing_docstrings()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup


setup(
    name='django_wtfd',
    version='0.1.1',
    description='Forcing developers to write docstrings.',
    author='Artem Kostiuk',
    author_email='postatum@gmail.com',
    long_description=open('README.md', 'r').read(),
    url='https://github.com/postatum/django-wtfd',
    download_url = 'https://github.com/postatum/django-wtfd/tarball/0.1.1',
    packages=[
        'django_wtfd',
        'django_wtfd.management',
        'django_wtfd.management.commands',
    ],
    keywords = ['docs', 'docstrings'],
    classifiers = [],
    zip_safe=False,
)
